// ORD Modules.scad
//These modules are used in the Optics Ray Diagram model.
//include<ORD Modules.scad>;

//Arrow-Triangle Module
module triangle (l, w, h){
   polyhedron(
               points=[ [0,0,0], //0
                       [l,0,0], //1
                       [l/2,w,0], //2
                       [0,0,h],  //3
                       [l,0,h], //4
                       [l/2,w,h] ], //5
    
               faces=[[0,1,2], 
                      [3,5,4], 
                      [0,2,5,3],
                      [0,3, 4, 1],
                      [1, 4, 5, 2]
    ]
               );}            

//Arrow Module
module arrow(size, height, thickness){
   union(){
      translate([-size/6, size*(3/4), 0]){
        triangle(size*(1/3), size*(1/4), height);}
      translate([-size/32, 0, 0]){
        cube([size/16, size*(3/4), height]);}
  }}

module imagearrow(a, b){
translate([imagelocation+((-imageheight)*(1/6)), imageheight*(3/4), 0]){
    rotate([0,0,a*180]){
  triangle((-imageheight)*(1/3), (-imageheight)*(1/4), outlineheight+2);}}//image arrow
translate([(imagelocation+(imageheight/32)),0,0]){
    rotate([0,0,a*-90]){
        imgdashedline(b*imageheight*(3/4), 2, outlineheight+2, (b*imageheight/16));}}//image dashed line
    }
    
//Outer Square Module
module outline(length, width, thickness, height){
    translate([length/2, width/2, 0]){
    translate([-length/2, -width/2, 0]){
        cube([thickness, width, height]);} //left vertical arm
    translate([length/2, -width/2, 0]){
        cube([thickness, width, height]);} //right vertical arm
    translate([-length/2, -width/2, 0]){
        cube([length, thickness, height]);} //bottom horizontal arm
    translate([-length/2, width/2, 0]){
        cube([length+thickness, thickness, height]);} //top horizontal arm
    }
}

//Braille Writer Module
module drawtext(text) {
	//Characters - Table is incomplete
	gs8chars = ",;':-.!?()$&@/^_`{}~#%*+<=>×·÷0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyzαβγδεζηθικλμνξοπρστυφχψωΓΔΘΛΞΠΣΥΦΨΩ";

	/* Chracter table defining 2x4 GS8 braille characters
	Adapted from: http://www.geocities.com/dinceraydin/djlcdsim/chartable.js
    GS8 chars from https://en.wikipedia.org/wiki/Gardner–Salinas_braille_codes
    GS8 Braille dot cell placement
    [ 1, 2,
      3, 4,
      5, 6,
      7, 8 ]
*/
	gs8char_table = [ 
                   [ 0, 0, 1, 0, 0, 0, 0, 0], //,
                   [ 0, 0, 1, 0, 1, 0, 0, 0], //;
                   [ 0, 0, 0, 0, 1, 0, 0, 0], //'
                   [ 0, 0, 1, 1, 0, 0, 0, 0], //:
                   [ 0, 0, 0, 0, 1, 1, 0, 0], //-
                   [ 0, 0, 1, 1, 0, 1, 0, 0], //.
                   [ 0, 0, 1, 1, 1, 0, 0, 0], //!
                   [ 0, 0, 1, 1, 1, 1, 0, 0], //?
                   [ 0, 0, 1, 0, 1, 0, 0, 1], //(
                   [ 0, 1, 0, 1, 0, 0, 1, 0], //)
                   [ 0, 1, 0, 1, 1, 1, 1, 0], //$
                   [ 1, 1, 1, 0, 1, 1, 0, 0], //&
                   [ 0, 1, 0, 0, 1, 0, 0, 1], //@
                   [ 1, 0, 0, 0, 0, 1, 1, 1], // \ 
                   [ 0, 0, 0, 1, 0, 0, 1, 0], //^
                   [ 0, 0, 1, 0, 0, 0, 0, 1], //_
                   [ 0, 0, 0, 0, 1, 0, 1, 1], //`
                   [ 1, 0, 1, 1, 1, 1, 1, 1], //{
                   [ 0, 1, 1, 1, 1, 1, 1, 1], //}
                   [ 0, 0, 0, 0, 0, 1, 1, 1], //~
                   [ 0, 1, 0, 1, 1, 1, 0, 0], //#
                   [ 0, 1, 1, 1, 1, 0, 0, 0], //%
                   [ 0, 1, 0, 1, 1, 0, 0, 0], //*
                   [ 0, 0, 1, 1, 0, 0, 1, 0], //+    
                   //[ 0, 1, 0, 0, 1, 0, 1, 1], // /  can't put this in the chars string  
                   [ 0, 0, 0, 1, 1, 0, 0, 1], //<    
                   [ 0, 0, 1, 1, 1, 1, 1, 1], //=    
                   [ 0, 0, 1, 0, 0, 1, 1, 0], //>    
                   [ 0, 0, 1, 0, 0, 0, 1, 1], //×    
                   [ 0, 0, 1, 1, 0, 0, 0, 1], //·    
                   [ 0, 0, 0, 1, 0, 0, 1, 1], //÷    
                   [ 0, 1, 0, 0, 1, 1, 0, 0], //0
                   [ 1, 0, 0, 0, 0, 1, 0, 0], //1
                   [ 1, 0, 1, 0, 0, 1, 0, 0], //2
                   [ 1, 1, 0, 0, 0, 1, 0, 0], //3
                   [ 1, 1, 0, 1, 0, 1, 0, 0], //4
                   [ 1, 0, 0, 1, 0, 1, 0, 0], //5
                   [ 1, 1, 1, 0, 0, 1, 0, 0], //6
                   [ 1, 1, 1, 1, 0, 1, 0, 0], //7
                   [ 1, 0, 1, 1, 0, 1, 0, 0], //8
                   [ 0, 1, 1, 0, 0, 1, 0, 0], //9
                   [ 1, 0, 0, 0, 0, 0, 1, 0], //A
                   [ 1, 0, 1, 0, 0, 0, 1, 0], //B
                   [ 1, 1, 0, 0, 0, 0, 1, 0], //C
                   [ 1, 1, 0, 1, 0, 0, 1, 0], //D
                   [ 1, 0, 0, 1, 0, 0, 1, 0], //E
                   [ 1, 1, 1, 0, 0, 0, 1, 0], //F
                   [ 1, 1, 1, 1, 0, 0, 1, 0], //G
                   [ 1, 0, 1, 1, 0, 0, 1, 0], //H
                   [ 0, 1, 1, 0, 0, 0, 1, 0], //I
                   [ 0, 1, 1, 1, 0, 0, 1, 0], //J
                   [ 1, 0, 0, 0, 1, 0, 1, 0], //K
                   [ 1, 0, 1, 0, 1, 0, 1, 0], //L
                   [ 1, 1, 0, 0, 1, 0, 1, 0], //M
                   [ 1, 1, 0, 1, 1, 0, 1, 0], //N
                   [ 1, 0, 0, 1, 1, 0, 1, 0], //O
                   [ 1, 1, 1, 0, 1, 0, 1, 0], //P
                   [ 1, 1, 1, 1, 1, 0, 1, 0], //Q
                   [ 1, 0, 1, 1, 1, 0, 1, 0], //R
                   [ 0, 1, 1, 0, 1, 0, 1, 0], //S
                   [ 0, 1, 1, 1, 1, 0, 1, 0], //T
                   [ 1, 0, 0, 0, 1, 1, 1, 0], //U
                   [ 1, 0, 1, 0, 1, 1, 1, 0], //V
                   [ 0, 1, 1, 1, 0, 1, 1, 0], //W
                   [ 1, 1, 0, 0, 1, 1, 1, 0], //X
                   [ 1, 1, 0, 1, 1, 1, 1, 0], //Y
                   [ 1, 0, 0, 1, 1, 1, 1, 0], //Z
                   [ 1, 0, 0, 0, 0, 0, 0, 0], //a
                   [ 1, 0, 1, 0, 0, 0, 0, 0], //b
                   [ 1, 1, 0, 0, 0, 0, 0, 0], //c
                   [ 1, 1, 0, 1, 0, 0, 0, 0], //d
                   [ 1, 0, 0, 1, 0, 0, 0, 0], //e
                   [ 1, 1, 1, 0, 0, 0, 0, 0], //f
                   [ 1, 1, 1, 1, 0, 0, 0, 0], //g
                   [ 1, 0, 1, 1, 0, 0, 0, 0], //h
                   [ 0, 1, 1, 0, 0, 0, 0, 0], //i
                   [ 0, 1, 1, 1, 0, 0, 0, 0], //j
                   [ 1, 0, 0, 0, 1, 0, 0, 0], //k
                   [ 1, 0, 1, 0, 1, 0, 0, 0], //l
                   [ 1, 1, 0, 0, 1, 0, 0, 0], //m
                   [ 1, 1, 0, 1, 1, 0, 0, 0], //n
                   [ 1, 0, 0, 1, 1, 0, 0, 0], //o
                   [ 1, 1, 1, 0, 1, 0, 0, 0], //p
                   [ 1, 1, 1, 1, 1, 0, 0, 0], //q
                   [ 1, 0, 1, 1, 1, 0, 0, 0], //r
                   [ 0, 1, 1, 0, 1, 0, 0, 0], //s
                   [ 0, 1, 1, 1, 1, 0, 0, 0], //t
                   [ 1, 0, 0, 0, 1, 1, 0, 0], //u
                   [ 1, 0, 1, 0, 1, 1, 0, 0], //v
                   [ 0, 1, 1, 1, 0, 1, 0, 0], //w
                   [ 1, 1, 0, 0, 1, 1, 0, 0], //x
                   [ 1, 1, 0, 1, 1, 1, 0, 0], //y
                   [ 1, 0, 0, 1, 1, 1, 0, 0], //z  
                   [ 1, 0, 0, 0, 0, 0, 0, 1], //α
                   [ 1, 0, 1, 0, 0, 0, 0, 1], //β
                   [ 1, 1, 1, 1, 0, 0, 0, 1], //γ
                   [ 1, 1, 0, 1, 0, 0, 0, 1], //δ
                   [ 1, 0, 0, 1, 0, 0, 0, 1], //ε
                   [ 1, 0, 0, 1, 1, 1, 0, 1], //ζ
                   [ 1, 0, 0, 1, 0, 1, 0, 1], //η
                   [ 1, 1, 0, 1, 0, 1, 0, 1], //θ
                   [ 0, 1, 1, 0, 0, 0, 0, 1], //ι
                   [ 1, 0, 0, 0, 1, 0, 0, 1], //κ
                   [ 1, 0, 1, 0, 1, 0, 0, 1], //λ
                   [ 1, 1, 0, 0, 1, 0, 0, 1], //μ
                   [ 1, 1, 0, 1, 1, 0, 0, 1], //ν
                   [ 1, 1, 0, 0, 1, 1, 0, 1], //ξ
                   [ 1, 0, 0, 1, 1, 0, 0, 1], //ο
                   [ 1, 1, 1, 0, 1, 0, 0, 1], //π
                   [ 1, 0, 1, 1, 1, 0, 0, 1], //ρ
                   [ 0, 1, 1, 0, 1, 0, 0, 1], //σ
                   [ 0, 1, 1, 1, 1, 0, 0, 1], //τ
                   [ 1, 0, 0, 0, 1, 1, 0, 1], //υ
                   [ 1, 1, 1, 0, 0, 0, 0, 1], //φ
                   [ 1, 1, 1, 0, 1, 1, 0, 1], //χ
                   [ 1, 1, 0, 1, 1, 1, 0, 1], //ψ
                   [ 0, 1, 1, 1, 0, 1, 0, 1], //ω
                   [ 1, 1, 1, 1, 0, 0, 1, 1], //Γ
                   [ 1, 1, 0, 1, 0, 0, 1, 1], //Δ
                   [ 1, 1, 0, 1, 0, 1, 1, 1], //Θ
                   [ 1, 0, 1, 0, 1, 0, 1, 1], //Λ
                   [ 1, 1, 0, 0, 1, 1, 1, 1], //Ξ
                   [ 1, 1, 1, 0, 1, 0, 1, 1], //Π
                   [ 0, 1, 1, 0, 1, 0, 1, 1], //Σ
                   [ 1, 0, 0, 0, 1, 1, 1, 1], //Υ
                   [ 1, 1, 1, 0, 0, 0, 1, 1], //Φ
                   [ 1, 1, 0, 1, 1, 1, 1, 1], //Ψ
                   [ 0, 1, 1, 1, 0, 1, 1, 1] //Ω       
//                 [ 0, 0, 0, 0, 0, 0, 0, 0], //template for additional characters     
    ];
    
	//Process string one character at a time
	for(itext = [0:len(text)-1]) {
		//Convert character to index
        ichar = search(text[itext],gs8chars,1)[0];
        val = gs8char_table[ichar];
        //Decode character - rows each character has 4 rows and 2 columns
        //Select value to draw from table
        for(icol = [0:1]) {
            for(irow = [0:3]) {
                // Retrieve bit to draw
               translate([cellspacing*itext+dotspacing*icol, dotspacing*irow, 0])
               if(val[irow*2+icol]==1) { sphere(radius);}
            }
        }
	}
}

module linemaker(length, linewidth, lineheight, segmentlength, spacelength){
    difference(){
    cube([length, linewidth, lineheight]);
    for(a = [0:((length)/(segmentlength+spacelength))]){
        translate([segmentlength+a*(segmentlength+spacelength), -0.01, -0.01]){
            cube([spacelength, linewidth+.02, lineheight+.02]);  
            }
        }
    }
}

module dashedline(length, unitlength, lineheight){
linemaker(length, linewidth, lineheight, 3*unitlength, unitlength);
}

module dottedline(length, unitlength, lineheight){
linemaker(length, linewidth, lineheight, unitlength, unitlength);
}

module solidline(length, unitlength, lineheight){
linemaker(length, linewidth, lineheight, unitlength, 0);
}

module realgrid(a){
    rotate([0,0,180]){
        cube([objectlocation*1.25, a, baseheight]);}//third quadrant
    translate([0,0,baseheight]){
        rotate([0,180,180]){
            cube([imagelocation*1.25, a, baseheight]);}}//fourth quadrant
    if (focalpoint*1.1 < objectheight*1.25){
           translate([-objectlocation*1.25, -a, 0]){
                outline((imagelocation*1.25)+(objectlocation*1.25), a+(objectheight*1.25), thickness, outlineheight);}
            cube([imagelocation*1.25, objectheight*1.25+(cellspacing*5), baseheight]);//first quadrant 
            translate([0,0,baseheight]){
                rotate([0,180,0]){
                    cube([objectlocation*1.25, objectheight*1.25+(cellspacing*5), baseheight]);}}//second quadrant
    } else {
           translate([-objectlocation*1.25, -a, 0]){
                outline((imagelocation*1.25)+(objectlocation*1.25), a+(focalpoint*1.1+1), thickness, outlineheight);}
            cube([imagelocation*1.25, focalpoint*1.1+(cellspacing*5), baseheight]);//first quadrant 
            translate([0,0,baseheight]){
                rotate([0,180,0]){
                    cube([objectlocation*1.25, focalpoint*1.1+(cellspacing*5), baseheight]);}}//second quadrant
    }
}
module virtgrid(){
//Grid and Outline
    if (imageheight/2 > focalpoint){    
        translate([imagelocation*1.5, -imageheight/2-focalpoint/3, 0]){
            cube([(-imagelocation*1.5)+(focalpoint*1.25), imageheight*1.75+focalpoint/3+(cellspacing*5), baseheight]);}//base
        translate([imagelocation*1.5, -imageheight/2-focalpoint/3, 0]){
            outline((-imagelocation*1.5)+(focalpoint*1.25), imageheight*1.75+focalpoint/3, thickness, outlineheight);}//outline
        translate([-thickness/2, -imageheight/2-focalpoint/3, 0]){
            cube([thickness, imageheight*1.75+focalpoint/3, outlineheight]);}//center vertical line
    } else {
        translate([imagelocation*1.5, -imageheight/2-focalpoint/1.5, 0]){
            cube([(-imagelocation*1.5)+(focalpoint*1.25), imageheight*1.75+focalpoint/1.5+cellspacing*5, baseheight]);}//base
        translate([imagelocation*1.5, -imageheight/2-focalpoint/1.5, 0]){
            outline((-imagelocation*1.5)+(focalpoint*1.25), imageheight*1.75+focalpoint/1.5, thickness, outlineheight);}//outline
        translate([-thickness/2, -imageheight/2-focalpoint/1.5, 0]){
            cube([thickness, imageheight*1.75+focalpoint/1.5, outlineheight]);}//center vertical line
    }


translate([imagelocation*1.5, -thickness/2, 0]){
    cube([(-imagelocation*1.5)+(focalpoint*1.25), thickness, outlineheight]);
}//center horizontal line
}

module virtrays(){
//Rays and Objects/Images
translate([-objectlocation, 0, 0]){
    arrow(objectheight, outlineheight+2, 1);
}//Object Arrow
translate([-objectlocation, objectheight-(thickness/2), 0]){
    dottedline(objectlocation+thickness, 3, lineheight);
}//middle ray from object to lens
translate([imagelocation, imageheight, 0]){
rotate([0, 0, theta]){
    solidline(pow((pow(-imagelocation, 2) + pow(imageheight, 2)), 1/2), 2, virtlineheight);
    }
translate([-imagelocation-objectlocation,-imageheight+objectheight,0]){
rotate([0, 0, theta]){
    solidline(pow((pow(-imagelocation/2+objectlocation, 2) + pow(objectheight+(imageheight/1.3)+focalpoint, 2)), 1/2), 2, lineheight);
    }}
}//center solid ray
translate([-objectlocation, objectheight, 0]){
rotate([0,0, theta2]){
    dashedline(pow((pow(totalheight, 2) + pow(objectlocation, 2)), 1/2), 2, lineheight);
    }
}//top ray from object to lens
rotate([0,0,180]){
translate([0, -objectheight-thickness/4, 0]){
    rotate([0, 0, theta3]){
        dottedline(pow((pow(totalheight, 2) + pow(imagelocation, 2)), 1/2), 3, virtlineheight);
    }}//middle ray from lens to image
translate([0, -objectheight+thickness, 0]){
    rotate([0, 0, 180+theta3]){
        dottedline(pow((pow(objectheight, 2) + pow(focalpoint, 2)), 1/2), 3, lineheight);
    }}//continuation of above ray
}
imagearrow(-1, 1);
translate([imagelocation, imageheight, 0]){
    dashedline(-imagelocation, 2, virtlineheight);
} //top line from lens to image
rotate([0,0,180])
translate([-focalpoint*1.25, -imageheight, 0]){
    dashedline(focalpoint*1.25, 2, lineheight);}
} //continuation of above line


module realrays(){
//Rays and Objects/Images
translate([-objectlocation, 0, 0]){
    arrow(objectheight, outlineheight+2, 1);
}//Object Arrow  
translate([-objectlocation, objectheight-(thickness/2), 0]){
    dottedline(objectlocation+thickness, 3, lineheight);
}//top left ray
translate([-objectlocation, objectheight, 0]){
rotate([0, 0, theta]){
    solidline(pow((pow(totallocation, 2) + pow(totalheight, 2)), 1/2), 2, lineheight);
    }
}//center solid ray
translate([-objectlocation, objectheight, 0]){
rotate([0,0, theta2]){
    dashedline(pow((pow(totalheight, 2) + pow(objectlocation, 2)), 1/2), 2, lineheight);
    }
}//bottom left ray
translate([0, objectheight-thickness/4, 0]){
    rotate([0, 0, theta3]){
        dottedline(pow((pow(totalheight, 2) + pow(imagelocation, 2)), 1/2), 3, lineheight);
    }
}//top right ray
imagearrow(1, -1);
translate([0, imageheight, 0]){
    dashedline(imagelocation, 2, lineheight);
}//bottom right ray
}

module imagetext(r, a) {
translate([imagelocation+dotspacing, r*cellspacing*2.2, 0]){
    rotate([0,0,180]){
    cube([5,dotspacing*3.5,outlineheight/2]);}}
translate([imagelocation-1, a, outlineheight/2]){
    rotate([0,0,270]){
    drawtext("i");}
}}//image

module lens(a){
//lens
difference(){
intersection() {
    translate([-focalpoint*2+lensthickness, 0, 0]){
        cylinder(h = outlineheight+.5, r = focalpoint*2, center = false, $fn = 40);}
    translate([focalpoint*2-lensthickness, 0, 0]){
        cylinder(h = outlineheight+.5, r = focalpoint*2, center = false, $fn = 40);}
    }//outer circle
intersection() {
    translate([-focalpoint*2+lensthickness, 0, 0]){
        cylinder(h = a*(outlineheight+2), r = focalpoint*2-thickness, center = false, $fn = 40);}
    translate([focalpoint*2-lensthickness, 0, 0]){
        cylinder(h = a*(outlineheight+2), r = focalpoint*2-thickness, center = false, $fn = 40);}
    }    
}//inner, subtracted circle
translate([-thickness/2, -focalpoint, 0]){
    cube([thickness, focalpoint*2, outlineheight+1.5]);}
}

module imgdashedline(length, unitlength, lineheight, linewidth){
linemaker(length, linewidth, lineheight, 3*unitlength, unitlength);
}

module focalpoint(a){
translate([a*focalpoint+(thickness/2), 0, 0]){
    cylinder(h = outlineheight + 2, r = 2*thickness, center = false, $fn = 40);}
}

module title(label, a, b){
    translate([-((objectlocation*1.25)+(b*-imagelocation*2))/2.25, focalpoint*1.3+(cellspacing*a*1.5), baseheight]){
        rotate([180,0,0]){
        drawtext(label);}}
    } 