//Optics Ray Diagram Model
//Chris Marshall
//UW-Whitewater
//Last Modified 7/26/16

include<ORD_Modules.scad>;

/*
This program draws a 3D Optics Ray Diagram.  This diagram can handle both real and virtual images and will change based off of inputted values for the object height, focal point, and object location.

    For example: an object of height=20, focal point=30 and object location=20 will produce a virtual image 60 units behind the lens with a height of 60.  
    
    an object of height=20, focal point=30, and object location=40 will produce an inverted real image 120 units from the lens with a height of 60.
      
    If you have a Dremel Idea Builder printer, the following are recommendations for printing to make sure the object fits on your bed:
        
        Object Height = 20
        Focal Point = 20
        Object Location = 40
        
        Object Height = 20
        Focal Point = 20
        Object Location = 32
        
        Object height = 15
        Focal Point = 20
        Object Location = 12
        
    In order to make sure the program works, keep your values reasonable.

This program requires the file ORD_Modules.scad in order to function.
*/

objectheight = 20; //the y value of the object
focalpoint = 20; //distance of the focal point from the lens
objectlocation = 40; //The distance of the object from the lens.
solidorhollow = "solid";

//For making braille that feels nice
radius = 1.0;
dotspacing = 2.4;
cellspacing = 6.2;
$fn = 20;
//The following values are used to make the code work, they don't need to be changed.
thickness = 1;
linewidth = thickness;
lensthickness = 8; //the thickness of the lens.  Note: this is for appearances only, this program uses the thin lens approximation.
outlineheight = 3;
baseheight = outlineheight/3;
lineheight = outlineheight + 2;
virtlineheight = lineheight/2;
//image equation (1/f)=(1/d)+(1/i)
imagelocation = (focalpoint*objectlocation)/(objectlocation-focalpoint);
//magnification equation m=-(i/d)
magnification = imagelocation/objectlocation;
imageheight = -magnification * objectheight;
theta = -atan(objectheight/objectlocation);
theta2 = -atan(objectheight/(objectlocation-focalpoint));
theta3 = -atan(objectheight/focalpoint);
totalheight = objectheight - imageheight;
totallocation = objectlocation + imagelocation;



        
//base and gridlines

if(objectlocation > focalpoint){
    
    translate([focalpoint+dotspacing, cellspacing*2.2, 0]){
        rotate([0,0,180]){
            cube([5,dotspacing*3.5,outlineheight/2]);}}
    translate([focalpoint-(dotspacing/2), 12, outlineheight/2]){
        rotate([0,0,270]){
        drawtext("f");}
        }//right focal point
    
    realrays();
    translate([10,0,0]){
        title("real image", 1, -1);
        title("converginng lens", 2.5, -1);}            
    imagetext(1, 12);
        if (-imageheight*1.25 > focalpoint){
            realgrid(-imageheight*1.25);
        } else {
            realgrid(focalpoint*1.25);}
            
    translate([-thickness/2, imageheight, 0]){
        cube([thickness, abs(-imageheight+objectheight), outlineheight+1.5]);}
    
    translate([-objectlocation*1.25,0,0]){
        cube([(objectlocation*1.25)+(imagelocation*1.25), thickness, outlineheight]);}
    
    translate([0,imageheight*1.25,0]){
        cube([thickness, (objectheight*1.25)+(-imageheight*1.25), outlineheight]);}
    
    focalpoint(-1);
    //focalpoint(-2);
    focalpoint(1);
        if (focalpoint*2 < objectlocation*1.25){
            focalpoint(-2);
        }else if (focalpoint*2 < imagelocation*1.25){
            focalpoint(2);}

} else {
    virtrays();
    virtgrid();
    imagetext(-0.35, -6.5);
    translate([-thickness/2, 0, 0]){
        cube([thickness, imageheight, outlineheight+1.5]);}
    focalpoint(-1);
    focalpoint(1);
    translate([focalpoint-dotspacing, -cellspacing*2.2, 0]){
            cube([5,dotspacing*3.5,outlineheight/2]);}
    translate([focalpoint-(dotspacing/2), -cellspacing*1.1, outlineheight/2]){
        rotate([0,0,270]){
        drawtext("f");}
    }//right focal point
    title("converging lens", 2.5, 1);
    title("virtual image", 1, 1);
}

if (solidorhollow == "solid"){
        lens(0);}
        else if (solidorhollow == "hollow"){
            lens(1);}
             
//Labels
translate([-focalpoint-dotspacing, -cellspacing*2.2, 0]){
    cube([5,dotspacing*3.5,outlineheight/2]);}
translate([-focalpoint-(dotspacing/2), -cellspacing*1.1, outlineheight/2]){
    rotate([0,0,270]){
        drawtext("f");}
}//left focal point

translate([-objectlocation-dotspacing, -cellspacing*2.2, 0]){
    cube([5,dotspacing*3.5,outlineheight/2]);}
translate([-objectlocation-1, -12, outlineheight/2]){
    drawtext("o");
}//object    